/*
 * Aspera, Inc.
 *
 * C++ Only.
 *
 * FaspOutputStream class.
 */
#pragma once

#include <string>
#include <vector>

#include <Poco/Thread.h>
#include <Poco/RunnableAdapter.h>
#include <Poco/Process.h>
#include <Poco/SharedPtr.h>
#include <Poco/PipeStream.h>

#include "Constants.h"
#include "FaspStreamLogger.h"

#ifdef WIN32

#ifdef FASPSTREAM_CPP_EXPORTS
#define FASPSTREAM_CPP_API __declspec(dllexport)
#else
#define FASPSTREAM_CPP_API __declspec(dllimport)
#endif

#else // NOT WIN32
#define FASPSTREAM_CPP_API
#endif

/**
 * Use the FaspOutputStream class to write faspstream data.  Note this class
 * does not inherit from any of the C++ std::ios stream classes, and
 * requires the user to implement a write() loop to send the data.
 */
class FaspOutputStream
{
public:
    /**
     * Initializes fasp output stream to use inPipe connected
     * to faspstream process. The faspstream process should read
     * from inPipe, and FaspOutputStream will write to it.
     */
    FASPSTREAM_CPP_API FaspOutputStream(const Poco::Pipe &inPipe, 
                                        const Poco::SharedPtr<Poco::ProcessHandle> ph,
                                        const FaspStreamLogger &logger);
    /**
     * Writes data to faspstream.
     *
     * Returns the number of bytes written, negative for error.
     *
     * This is a blocking function.
     *
     * Throws a FaspStreamException if there is an error writing to the
     * faspstream.
     *
     * NOTE: You may also encounter a SIGPIPE if the faspstream process
     *       exited prematurely (perhaps due to an error).
     *
     * @param buffer data to write to faspstream
     * @param length number of bytes to write in buffer
     * @return number of bytes written, negative for error
     */
    FASPSTREAM_CPP_API void write(const char* buffer, int length);
    /**
     * Closes the pipe to the faspstream process.
     */
    FASPSTREAM_CPP_API void close();
    FASPSTREAM_CPP_API void abort();
    FASPSTREAM_CPP_API void cleanup();

private:
    Poco::Pipe _inPipe;
    Poco::SharedPtr<Poco::ProcessHandle> _ph;
    FaspStreamLogger _logger;
};
