/*
 * Aspera, Inc.
 *
 * C++ only.
 *
 * A FaspStreamServer is used to listen for a faspstream
 * client to connect to it and send/recv data.
 *
 * You can set many of the faspstream configuration parameters
 * using the FaspStreamServer's interface.
 *
 * See the samples for typical usage.
 */
#pragma once

#include <string>
#include <vector>

#include <Poco/Process.h>
#include <Poco/SharedPtr.h>
#include <Poco/Thread.h>

#include "Constants.h"
#include "FaspInputStream.h"
#include "FaspOutputStream.h"
#include "FaspConnectionChannel.h"
#include "ChannelHandler.h"
#include "FaspStreamLogger.h"

#ifdef WIN32

#ifdef FASPSTREAM_CPP_EXPORTS
#define FASPSTREAM_CPP_API __declspec(dllexport)
#else
#define FASPSTREAM_CPP_API __declspec(dllimport)
#endif

#else // NOT WIN32
#define FASPSTREAM_CPP_API
#endif

/**
 * Use the FaspStreamServer class to wait for stream connections
 * and then receive (read) or send (write) a stream of data.  Note this class
 * does not inherit from any of the C++ std::ios stream classes, and
 * requires the user to implement a read()/write() loop.
 */
class FaspStreamServer : public FaspStreamLogger
{
public:
    /**
     * Does basic initialization of server's fields, sets defaults, etc.
     *
     * The channel argument is optional and is only used when you would like
     * to share a TCP connection so that our internal code can tunnel fasp
     * control data on the TCP connection you share.
     *
     * IMPORTANT: If you use the channel argument, the channel's destructor
     *            must fire *after* the server's destructor. Otherwise, the
     *            background thread that does the tunneling may cause a seg
     *            fault.
     *
     * @param appId a string identifier which is unique to this application
     * @param channel an implementation of the FaspConnectionChannel iface
     */
    FASPSTREAM_CPP_API FaspStreamServer(const std::string& appId,
        FaspConnectionChannel *channel = NULL);
    /**
     * Starts the faspstream process in server mode, listening for an
     * incoming client connection.
     *
     * This method will not block to wait for an incoming connection, but
     * returns immediately. (The faspstream process, meanwhile, is waiting
     * for the incoming connection.)
     *
     * You will not block until you try to do a read or write on the
     * input/output streams. (This is because the faspstream process is
     * still waiting for a client to connect and is not reading or writing
     * from its stdin/stdout.)
     *
     * The killChild parameter is currently not used.
     *
     * @return whether (true) or not (false) the sub-process started correctly
     */
    FASPSTREAM_CPP_API bool startServer(bool killChild = false);
    /**
     * Use these to get the underlying input and output streams
     * connected to the faspstream process.
     *
     * IMPORTANT: You should call these every time you try to do a read/write.
     *            We do a check each time you invoke these to ensure
     *            the faspstream is still open.
     *
     * NOTE: If you are doing a "send", you need the output stream. If
     *       you are doing a "recv", you need the input stream.
     */
    FASPSTREAM_CPP_API FaspInputStream &getFaspInputStream();
    FASPSTREAM_CPP_API FaspOutputStream &getFaspOutputStream();
    /**
     * IMPORTANT: You should call this before the destructor fires. If the
     *            faspstream process is still working and destructor is
     *            called, it will be killed.
     *
     * Closes input and output streams and waits for faspstream process to
     * gracefully exit.
     *
     * If you shared a TCP socket for fasp control data tunneling, the
     * background thread will be terminated, and under normal conditions,
     * your channel's close method will be invoked.
     *
     * Throws a FaspStreamException if the exit status of the faspstream
     * process is non-zero.
     */
    FASPSTREAM_CPP_API void cleanup();
    /**
     * Kills faspstream process if it is still alive. Does any remaining
     * clean up.
     */
    FASPSTREAM_CPP_API ~FaspStreamServer();

    /*---------- Faspstream Configuration ----------*/

    FASPSTREAM_CPP_API void setFaspstreamPath(
        const std::string& faspstreamPath);
    FASPSTREAM_CPP_API std::string getFaspstreamPath() const;
    FASPSTREAM_CPP_API int getPort() const;
    FASPSTREAM_CPP_API void setPort(int tcpPort);
    FASPSTREAM_CPP_API int getMaxMemory() const;
    FASPSTREAM_CPP_API void setMaxMemory(int maxMem);
    FASPSTREAM_CPP_API std::string getUsername() const;
    FASPSTREAM_CPP_API int getUdpPort() const;
    FASPSTREAM_CPP_API void setUdpPort(int port);
    FASPSTREAM_CPP_API int getDatagramSize() const;
    FASPSTREAM_CPP_API void setDatagramSize(int size);
    FASPSTREAM_CPP_API int getChunkSize() const;
    FASPSTREAM_CPP_API void setChunkSize(int size);
    FASPSTREAM_CPP_API std::string getHostname() const;
    FASPSTREAM_CPP_API void setHostname(const std::string& name);
    FASPSTREAM_CPP_API int getRateTarget() const ;
    FASPSTREAM_CPP_API void setRateTarget(int target);
    FASPSTREAM_CPP_API FASPSTREAM_CIPHER_TYPE getCipherType() const;
    FASPSTREAM_CPP_API void setCipherType(FASPSTREAM_CIPHER_TYPE type);
    FASPSTREAM_CPP_API bool isLockTarget() const;
    FASPSTREAM_CPP_API void setLockTarget(bool target);
    FASPSTREAM_CPP_API int getRateMinimum() const;
    FASPSTREAM_CPP_API void setRateMinimum(int minimum);
    FASPSTREAM_CPP_API bool isLockMinimum() const;
    FASPSTREAM_CPP_API void setLockMinimum(bool lockMin);
    FASPSTREAM_CPP_API FASPSTREAM_POLICY_TYPE getPolicy() const;
    FASPSTREAM_CPP_API void setPolicy(FASPSTREAM_POLICY_TYPE newPolicy);
    FASPSTREAM_CPP_API bool isLockPolicy() const;
    FASPSTREAM_CPP_API void setLockPolicy(bool lock);
    FASPSTREAM_CPP_API int getAggressiveness() const;
    FASPSTREAM_CPP_API void setAggressiveness(int aggressive);
    FASPSTREAM_CPP_API int getForceSendMilliseconds() const;
    FASPSTREAM_CPP_API void setForceSendMilliseconds(int milliseconds);
    FASPSTREAM_CPP_API void setTransferMode(FASPSTREAM_TRANSFER_MODE_TYPE transferMode);
    FASPSTREAM_CPP_API FASPSTREAM_TRANSFER_MODE_TYPE getTransferMode() const;
    FASPSTREAM_CPP_API void setLogDirectory(std::string logDirectory);
    FASPSTREAM_CPP_API std::string getLogDirectory() const;
    FASPSTREAM_CPP_API void setCommandToExecute(std::string commandToExecute);
    FASPSTREAM_CPP_API std::string getCommandToExecute() const;
    FASPSTREAM_CPP_API void setLicense(std::string commandToExecute);
    FASPSTREAM_CPP_API std::string getLicense() const;
    FASPSTREAM_CPP_API void setAcceptTimeout(int timeout);
    FASPSTREAM_CPP_API int getAcceptTimeout() const;


    /**
     * This method can be used to generate the command line arguments
     * with which the faspstream child process will be invoked, it can be
     * useful for debugging purposes but is not normally necessary.
     */
    std::vector<std::string> getCommandline() const;

    /**  Method for registering a logging callback
      *
      *  When a new log entry is available the function specified will
      *  be called (signature: void return, const char* argument).
      *  (Will not log if NULL, as is by default)
      *
      *  @param log_callback pointer to log function you have implemented
      *
      *  The simplest implementation of the callback might look like this:
      *  <code><br/><br/>
      *      static void log_function( char const *s ) {<br/>
      *  &nbsp;  fprintf( stderr, "%s\n", s );<br/>
      *      }
      *  </code>
      */
    void registerLogCallback(log_callback_f log_function);
    virtual void log(std::string logEntry);

private:
    /*
     * Private members are mostly configuration settings
     */
    Poco::SharedPtr<Poco::ProcessHandle> _ph;
    Poco::Pipe _inPipe;
    Poco::Pipe _outPipe;
    Poco::Pipe _errPipe;
    FaspInputStream _fis;
    FaspOutputStream _fos;

    int _port;
    int _datagramSize;
    int _chunkSize;
    std::string _hostname;
    int _rateTarget;
    FASPSTREAM_CIPHER_TYPE _cipherType;
    bool _lockTarget;
    int _rateMinimum;
    bool _lockMinimum;
    FASPSTREAM_POLICY_TYPE _policy;
    bool _lockPolicy;
    int _aggressiveness;
    int _udpPort;
    int _maxMemory;
    std::string _faspstreamPath;
    std::string _errorString;
    bool _subprocessDied;
    int _forceSendMilliseconds;
    FASPSTREAM_TRANSFER_MODE_TYPE _transferMode;
    std::string _logDirectory;
    std::string _commandToExecute;
    std::string _appId;
    std::string _license;
    int _acceptTimeout;
    /*
     * Faspstream channel / out-of-band data
     */
    FaspConnectionChannel* _channel;
    Poco::Thread* _thread;
    ChannelHandler* _handler;

    void invoke_faspstream();
    void startChannelSocketListener();
    void check_faspstream();

    log_callback_f _log_callback;
    void logStdErr();
};
