/*
 * Aspera, Inc.
 *
 * This is a C wrapper around the FaspStreamServer.
 */

#include "fasp_stream_constants.h"

#ifndef ASPERA_FASP_STREAM_SERVER_WRAPPER_H
#define ASPERA_FASP_STREAM_SERVER_WRAPPER_H

#include <stdlib.h>
#include <fasp_stream_channel.h>

#ifdef WIN32

#ifdef FASPSTREAM_CPP_EXPORTS
#define FASPSTREAM_C_API __declspec(dllexport)
#else
#define FASPSTREAM_C_API __declspec(dllimport)
#endif

#else /* NOT WIN32 */
#define FASPSTREAM_C_API
#endif

#ifdef __cplusplus
extern "C" {
#endif

struct fasp_stream_server;
typedef struct fasp_stream_server fasp_stream_server_t;

/**
 * Wrapper around FaspStreamServer constructor.
 *
 * @param app_id string id that is unique to this app
 * @param fss out parameter for new fasp stream server
 * @return status, non-zero on error
 */
FASPSTREAM_C_API
int
fasp_stream_server_new(char *app_id, fasp_stream_server_t **fss);

/**
 * Wrapper around FaspStreamServer constructor for tunneling.
 *
 * @param app_id string id that is unique to this app
 * @param fss out parameter for new fasp stream server
 * @param wrapper the socket to use for tunneling fasp control data
 * @return status, non-zero on error
 */
FASPSTREAM_C_API
int
fasp_stream_channel_server_new(char *app_id, fasp_stream_server_t **fss,
    fasp_stream_socket_wrapper_t *sock);

/**
 * Wrapper around FaspStream abort(). This will kill the
 * faspstream process in case an error has occurred or process is "hung".
 * This is intended to be used in error / exception handling code.
 *
 * @param fss the fasp stream server to abort
 * @return non-zero on failure
 */
FASPSTREAM_C_API
int
fasp_stream_server_abort(fasp_stream_server_t *fss);

/**
 * Wrapper around FaspStream abort(). This will kill the
 * faspstream process in case an error has occurred or process is "hung".
 * This is intended to be used in error / exception handling code.
 *
 * @param fss the fasp stream server to clean up
 * @return non-zero on failure
 */

FASPSTREAM_C_API
int
fasp_stream_server_cleanup(fasp_stream_server_t *fss);

/**
 * Wrapper around FaspStreamServer destructor.
 *
 * IMPORTANT: Call cleanup before calling destroy. If the faspstream
 *            process is still working and destroy is called, it will
 *            be killed, and may not send/recv all data.
 *
 * @param fss the fasp stream server to destroy / free
 * @return non-zero on failure
 */
FASPSTREAM_C_API
int
fasp_stream_server_destroy(fasp_stream_server_t *fss);

/**
 * Wrapper around FaspStreamServer::start_server.
 *
 * @param fss fasp stream server
 * @param kill_child
 * @return non-zero on failure
 */
FASPSTREAM_C_API
int
fasp_stream_server_start_server(fasp_stream_server_t *fss, int kill_child);

/**
 * Wrapper around writing to FaspStreamServer's output stream. The
 * transfer mode must be FASPSTREAM_TRANSFER_TYPE_SEND (fasp_stream_enums.h) -
 * set it with fasp_stream_server_set_transfer_mode. Otherwise, the child
 * fasp stream process will ignore what you write. Always writes entire buffer
 * to output stream unless there's an error.
 *
 * @param fss fasp stream server
 * @param buff buffer to write to output stream
 * @param len length of buffer
 * @return non-zero on error
 */
FASPSTREAM_C_API
int
fasp_stream_server_write(fasp_stream_server_t *fss, char *buff, size_t len);

/**
 * Wrapper around reading from FaspStreamServer's input
 * stream. The transfer mode must be FASPSTREAM_TRANSFER_TYPE_RECEIVE
 * (fasp_stream_enums.h) - set it with fasp_stream_server_set_transfer_mode).
 * Otherwise, the child fasp stream process won't provide anything for you to
 * read.
 *
 * @param fss fasp stream server
 * @param buff buffer to receive incoming data
 * @param len length of buffer
 * @return number of bytes read, zero means EOF, negative means error
 */
FASPSTREAM_C_API
int
fasp_stream_server_read(fasp_stream_server_t *fss, char *buff, size_t len);

/* ---------- GETTERS / SETTERS ---------- */

FASPSTREAM_C_API
const char * fasp_stream_server_get_username(fasp_stream_server_t *fss);

FASPSTREAM_C_API
int fasp_stream_server_get_max_memory(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_max_memory(fasp_stream_server_t *fss, int max_mem);

FASPSTREAM_C_API
int
fasp_stream_server_get_udp_port(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_udp_port(fasp_stream_server_t *fss, int port);

FASPSTREAM_C_API
int
fasp_stream_server_get_port(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_port(fasp_stream_server_t *fss, int tcp_port);

FASPSTREAM_C_API
int
fasp_stream_server_get_datagram_size(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_datagram_size(fasp_stream_server_t *fss, int size);

FASPSTREAM_C_API
int
fasp_stream_server_get_chunk_size(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_chunk_size(fasp_stream_server_t *fss, int size);

FASPSTREAM_C_API
const char *
fasp_stream_server_get_hostname(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_hostname(fasp_stream_server_t *fss, char *name);

FASPSTREAM_C_API
int
fasp_stream_server_get_rate_target(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_rate_target(fasp_stream_server_t *fss, int target);

FASPSTREAM_C_API
int
fasp_stream_server_get_cipher_type(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_cipher_type(fasp_stream_server_t *fss, int type);

FASPSTREAM_C_API
int
fasp_stream_server_is_lock_target(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_lock_target(fasp_stream_server_t *fss, int target);

FASPSTREAM_C_API
int
fasp_stream_server_get_rate_minimum(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_rate_minimum(fasp_stream_server_t *fss, int minimum);

FASPSTREAM_C_API
int
fasp_stream_server_is_lock_minimum(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_lock_minimum(fasp_stream_server_t *fss, int lock_min);

FASPSTREAM_C_API
int
fasp_stream_server_get_policy(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_policy(fasp_stream_server_t *fss, int new_policy);

FASPSTREAM_C_API
int
fasp_stream_server_is_lock_policy(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_lock_policy(fasp_stream_server_t *fss, int lock);

FASPSTREAM_C_API
int
fasp_stream_server_get_aggressiveness(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_aggressiveness(fasp_stream_server_t *fss,
    int aggressive);

FASPSTREAM_C_API
const char *
fasp_stream_server_get_faspstream_path(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_faspstream_path(fasp_stream_server_t *fss,
    char *path);

FASPSTREAM_C_API
int
fasp_stream_server_get_force_send_milliseconds(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_force_send_milliseconds(fasp_stream_server_t *fss,
    int milliseconds);

FASPSTREAM_C_API
void
fasp_stream_server_set_transfer_mode(fasp_stream_server_t *fss,
    int transfer_mode);

FASPSTREAM_C_API
int
fasp_stream_server_get_transfer_mode(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_log_directory(fasp_stream_server_t *fss,
    char *log_directory);

FASPSTREAM_C_API
const char *
fasp_stream_server_get_log_directory(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_command_to_execute(fasp_stream_server_t *fss,
    char *command_to_execute);

FASPSTREAM_C_API
const char *
fasp_stream_server_get_command_to_execute(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_license(fasp_stream_server_t *fss,
char *license);

FASPSTREAM_C_API
const char *
fasp_stream_server_get_license(fasp_stream_server_t *fss);

FASPSTREAM_C_API
int
fasp_stream_server_get_accept_timeout(fasp_stream_server_t *fss);

FASPSTREAM_C_API
void
fasp_stream_server_set_accept_timeout(fasp_stream_server_t *fss,
    int timeout);

/**  Function for registering a logging callback
  *
  *  When a new log entry is available the function specified will
  *  be called (signature: void return, const char* argument).
  *  (Will not log if NULL, as is by default)
  *
  *  @param log_callback pointer to log function you have implemented
  *
  *  The simplest implementation of the callback might look like this:
  *  <code><br/><br/>
  *      static void log_function( char const *s ) {<br/>
  *  &nbsp;  fprintf( stderr, "%s\n", s );<br/>
  *      }
  *  </code>
  */

FASPSTREAM_C_API
void
fasp_stream_server_register_log_callback(fasp_stream_server_t *fss, log_callback_f log_function );

#ifdef __cplusplus
};
#endif

#endif

