/*
 * Aspera, Inc.
 *
 * Example server that uses the C wrapper for the FaspStream
 * SDK.
 *
 * Usage:
 *
 *   ./server > outfile
 *
 * Uses mostly default settings. See setup_server.
 */

#include <stdio.h>
#include <stdlib.h>
#include <fasp_stream_server.h>
#include <fasp_stream_constants.h>
#ifdef WIN32
        #include <io.h>
        #include <fcntl.h>
#endif

static
void
log_msg(const char *msg)
{
    fprintf(stderr, "%s\n", msg);
}

static
int
setup_server(fasp_stream_server_t **out)
{
    fasp_stream_server_t *fss;
    int ret;
    /*
     * Create the server
     */
    ret = fasp_stream_server_new("server-id", &fss);
    if (ret) {
        log_msg("Error creating server");
        goto fail;
    }
    /*
     * Configure server
     * ----------------
     *
     * Path to faspstream binary (not the SDK library).
     */
    fasp_stream_server_set_faspstream_path(fss, FASPSTREAM_BINARY);
    /*
     * Set transfer mode to RECEIVE (server will receive data from client)
     */
    fasp_stream_server_set_transfer_mode(fss,
        FASPSTREAM_TRANSFER_MODE_TYPE_DIRECT_RECEIVE);
    /*
     * Rate target, in Kbps (100 Mbps)
     */
    fasp_stream_server_set_rate_target(fss, 100000);
    /*
     * Set port to accept TCP connection on
     */
    fasp_stream_server_set_port(fss, 4444);
    /*
     * Set UDP port for the fasp transfer
     */
    fasp_stream_server_set_udp_port(fss, 33001);
    /*
     * Register logging function callback
     */
    fasp_stream_server_register_log_callback(fss, &log_msg);

    /*
     *  Set force send interval on the server side
     *  It must match the client
     */
    fasp_stream_server_set_force_send_milliseconds(fss, 1000);

    *out = fss;

    return 0;

fail:
    return ret;
}

static
int
write_bytes(char *buff, int len, int total_bytes_received)
{
    int ret;
    /*
     * Write bytes to stdout
     */
    ret = fwrite(buff, 1, len, stdout);
    if (ret != len) {
        log_msg("Trouble writing to stdout?");
        return -1;
    }
    /*
     * Write progress to stderr
     */
    fprintf(stderr, "\rBytes Received: %d", total_bytes_received);
    fflush(stderr);

    return 0;
}

int
main() {
    fasp_stream_server_t *fss;
    int ret;
    int count;
    int total_bytes_received;
    char buff[512];
#ifdef WIN32
    /*
     * Required on Windows for binary files.
     */
    _setmode(fileno(stdout), O_BINARY);
#endif
    /*
     * Create and set up server
     */
    ret = setup_server(&fss);
    if (ret)
        goto fail1;
    /*
     * Listen for client to connect
     */
    log_msg("Waiting for client to connect...");
    ret = fasp_stream_server_start_server(fss, 0);
    if (ret) {
        log_msg("Error listening for client");
        goto fail2;
    }
    /*
     * Client connected
     */
    total_bytes_received = 0;
    while ((count = fasp_stream_server_read(fss, buff, sizeof(buff))) > 0) {
        /*
         * Read bytes from client, and write to stdout
         */
        total_bytes_received += count;
        if (write_bytes(buff, count, total_bytes_received))
            break;
    }

    fprintf(stderr, "\n");
    /*
     * Clean up
     */
    ret = fasp_stream_server_cleanup(fss);
    if (ret) {
        log_msg("Error cleaning up server");
        goto fail2;
    }

    goto out;

out:
fail2:
    /*
     * Destroy server
     */
    ret = fasp_stream_server_destroy(fss);
    if (ret) {
        log_msg("Error destroying server");
        abort();
    }
fail1:
    return ret;
}
