/*
 * Aspera, Inc.
 *
 * Client.cpp
 *
 * Shows how to connect to a faspstream server and send it a stream
 * of bytes from stdin.
 *
 * Usage:
 *
 *   ./Client < in.file
 */
#include <iostream>
#include <fstream>
#include <Constants.h>
#include <FaspOutputStream.h>
#include <FaspStreamClient.h>
#include <FaspStreamException.h>
#ifdef WIN32
        #include <io.h>
        #include <fcntl.h>
#endif

static
void
setupStreams() {
#ifdef WIN32
    /*
     * Required on Windows for binary files
     */
    _setmode(fileno(stdin), O_BINARY);
    _setmode(fileno(stdout), O_BINARY);
#endif
    std::cin.exceptions(std::ifstream::badbit);
    std::cout.exceptions(std::ifstream::badbit);
}

static
void
log_msg(const char *msg)
{
    fprintf(stderr, "%s\n", msg);
}

int main()
{
    char data[512];
    setupStreams();
    try {
        /*
         * Initialize faspstream client
         */
        FaspStreamClient fsc("Unique-ID-For-a-Client");
        /*
         * Configure client
         * ----------------
         *
         * Set target rate to 100 Mbps
         */
        fsc.setRateTarget(100000);
        /*
         * Set the port to connect to
         */
        fsc.setPort(4444);
       /*
        * Flush internal buffer after 1 second.
        */
        fsc.setForceSendMilliseconds(1000);
        /*
         * Set the UDP Port for the fasp transfer
         */
        fsc.setUdpPort(33001);

        /*
         * Register logging function callback
         */
        fsc.registerLogCallback(log_msg);

        /*
         * Connect to server
         * -----------------
         */
        fsc.setTransferMode(FASPSTREAM_TRANSFER_MODE_TYPE_DIRECT_SEND);

        fsc.connect("localhost");
        std::cerr << "Connected to server, sending data..."
                  << std::endl;
        /*
         * Send bytes from stdin
         */

        long totalSent = 0;
        while (std::cin) {
            long sent;
            /*
             * Read from stdin
             */
            std::cin.read(data, sizeof(data));
            sent = std::cin.gcount();
            totalSent += sent;
            /*
             * Write to fasp stream
             */
            fsc.getFaspOutputStream().write(data,
                static_cast<int>(sent));

            std::cerr << "Bytes Sent: " << totalSent << "\r";
            std::cerr.flush();
        }
        /*
         * Done; clean up client.
         */
        fsc.cleanup();

        std::cerr << std::endl
                  << "Transfer Complete!"
                  << std::endl;
        std::cerr << "Bytes Sent: "
                  << totalSent
                  << std::endl;

    }
    catch (std::ifstream::failure &e) {
        std::cerr << "Error reading from stdin: "
                  << std::endl
                  << e.what();
    }
    catch (std::exception &e) {
        std::cerr << "A standard exception occurred:" << std::endl;
        std::cerr << e.what();
    }
    catch (...)  {
        std::cerr << "An unknown exception ocurred! ";
        std::cerr << "(No further information is available)";
    }

    return 0;
}
