//
// © Copyright IBM Corp. 2015
//
// This program behaves similarly to running the following faspstream command:
//   faspstream --force-send-after=10 -i/Users/peter/.ssh/peter --exec="ruby faspstream_exec.rb --log --debug" --user=peter -P33001 -L/temp --mode=recv localhost
//
// It expects the faspstream_exec.rb script to be in the SSH root. The script is
// located on SVN in fasp/trunk/fasp3.
//
// On a Posix system, we can exec bash to do a similar thing:
//   faspstream --force-send-after=10 -i/Users/peter/.ssh/peter --exec=bash --user=peter -P33001 -L/temp --mode=recv localhost
//
// After the program connects successfully, send commands to execute on the
// remote host through stdin (console) - e.g. "ls" or "curl www.example.com".
// Send "exit" to notify the script when finished. Then send EOF (control-d)
// to terminate the program.
//
// TODO add fasp params to command line
//

#include <Constants.h>
#include <FaspOutputStream.h>
#include <FaspStreamClient.h>
#include <pthread.h>
#include <unistd.h>
#include <stdio.h>

/*
 * Read from FASP stream and write to console (stdout). Runs in its own thread.
 */
void* faspRecvThreadLoop(void* ptr) {
    FaspStreamClient* client = (FaspStreamClient*)ptr;
    ssize_t totalRead = 0;
    char buf[16384];
    for (;;) {
        // Read available bytes from FASP
        ssize_t bytesRead = client->getFaspInputStream().read(buf, 0, sizeof(buf));
        totalRead += bytesRead;
        if (bytesRead == 0) {
            fprintf(stdout, "End of FASP receive stream\n");
            break;
        } else if (bytesRead < 0) {
            fprintf(stderr, "Error reading FASP receive stream\n");
            break;
        }
        fprintf(stdout, "Total bytes read: %ld\n", totalRead);
        // Print in console
        //write(STDOUT_FILENO, buf, bytesRead);
        //fflush(stdout);
    }
    return 0;
}

int main() {
#ifdef WIN32
    // Required on Windows for binary files
    _setmode(_fileno(stdin), O_BINARY);
#endif
    
    try {
        // Actually, faspstream doesn't currently check ASPERA_SCP_LICENSE
        //if (setenv("ASPERA_SCP_LICENSE", "/Library/Aspera/etc/aspera-license", 1) != 0)
        //    fprintf(stderr, "Error setting ASPERA_SCP_LICENSE\n");
        if (setenv("ASPERA_SCP_PASS", "aspera", 1) != 0)
            fprintf(stderr, "Error setting ASPERA_SCP_PASS\n");
      
        FaspStreamClient client("cpp_exec");
        //client.setRateTarget(100000);
        //client.setUdpPort(33001);
        //client.setPort(33001);
        client.setFaspstreamPath("/Library/Aspera/bin/faspstream");
        client.setForceSendMilliseconds(10);
        client.setSSHUser("peter");
        client.setPrivateKeyFile("/Users/peter/.ssh/peter");
        client.setCommandToExecute("ruby faspstream_exec.rb --log --debug");
        //client.setCommandToExecute("bash");
        client.setLogDirectory("/temp");
        client.setTransferMode(FASPSTREAM_TRANSFER_MODE_TYPE_SSH_RECEIVE);
        
        /*
         * Connect
         */
        fprintf(stdout, "Connecting to server\n");
        if (client.connect("localhost")) {
            fprintf(stdout, "Connected!\n");
        } else {
            fprintf(stderr, "Error connecting to server\n");
            exit(EXIT_FAILURE);
        }

        /*
         * Receive thread
         */
        pthread_t thread;
        int result = pthread_create(&thread, NULL, faspRecvThreadLoop, (void*) &client);
        if (result) {
            fprintf(stderr, "pthread_create() failed: %d\n", result);
            exit(EXIT_FAILURE);
        }
        
        /*
         * Read stdin loop
         */
        ssize_t totalBytesWritten = 0;
        char buf[16384];
        for (;;) {
            ssize_t bytesRead = read(STDIN_FILENO, buf, sizeof(buf));
            if (bytesRead == 0) {
                fprintf(stdout, "End of input stream\n");
                break;
            } else if (bytesRead < 0) {
                fprintf(stderr, "read() error\n");
                break;
            }
            // [FIXME?] FaspOutputStream::write doesn't return anything currently
            //ssize_t bytesWritten = client.getFaspOutputStream().write(buf, bytesRead);
            //if (bytesWritten < 0) {
            //    fprintf(stderr, "Error writing to FASP stream\n");
            //} else {
            //    totalBytesWritten += bytesWritten;
            //    fprintf(stderr, "Bytes Sent: %ld\n", totalBytesWritten);
            //}
            client.getFaspOutputStream().write(buf, bytesRead);
            totalBytesWritten += bytesRead;
            fprintf(stderr, "Bytes Sent: %ld\n", totalBytesWritten);
        }
        client.getFaspOutputStream().close();
        
        // Wait for write thread to end
        // [FIXME] Remote is not shutting down when exec'd command ends
        fprintf(stdout, "Waiting for FASP receive thread to end\n");
        pthread_join(thread, NULL);
    } catch (std::exception &e) {
        fprintf(stderr, "Exception: %s\n", e.what());
        exit(EXIT_FAILURE);
    } catch (...)  {
        fprintf(stderr, "Unknown exception\n");
        exit(EXIT_FAILURE);
    }

    fprintf(stdout, "Exiting\n");
    fflush(stderr);
    fflush(stdout);
    exit(EXIT_SUCCESS);
}
