/*
 * Aspera, Inc.
 *
 * Client.cpp
 *
 * Shows how to use a single TCP connection for application-specific
 * data and fasp control data. Reads data from stdin and sends to
 * faspstream server.
 *
 * Usage:
 *
 *   ./Client < in.file
 */
#include <iostream>
#include <fstream>
#include <Constants.h>

#include <Poco/Net/SocketAddress.h>
#include <Poco/Net/StreamSocket.h>
#include <Poco/Net/NetException.h>
#include <Poco/Net/ServerSocket.h>

#include <FaspOutputStream.h>
#include <FaspStreamClient.h>
#include <FaspStreamException.h>
#ifdef WIN32
        #include <io.h>
        #include <fcntl.h>
#endif

/*
 * Configure these for the remote server.
 */
#define REMOTE_HOST "localhost"
#define REMOTE_PORT 4444

/*
 * To share a TCP connection with faspstream, you must implement
 * the FaspConnectionChannel interface, in FaspConnectionChannel.h.
 *
 * This implementation uses a Poco stream socket.
 *
 * Almost identical to implemenation in Server.cpp. See comments there
 * for more details.
 */
class SocketForwardingChannel : public FaspConnectionChannel {

private:
    Poco::Net::StreamSocket _remoteFaspstreamSocket;
        
public:

    SocketForwardingChannel(std::string hostname, int port)
    {
        /*
         * Connect to server
         */
        Poco::Net::SocketAddress sa(hostname, port);
        _remoteFaspstreamSocket = Poco::Net::StreamSocket(sa);
        _remoteFaspstreamSocket.setReceiveTimeout(Poco::Timespan(100000));
    }

    int write(const char *data, size_t size)
    {
        try {
            return _remoteFaspstreamSocket.sendBytes(data, size);
        }
        catch (Poco::Net::ConnectionResetException&) {
            throw FaspChannelClosedException();
        }
    }

    int read(char* buffer, size_t size) {
        try {
            return _remoteFaspstreamSocket.receiveBytes(buffer, size);
        }
        catch (Poco::TimeoutException&) {
            throw FaspChannelTimeoutException();
        }
        catch (Poco::Net::ConnectionResetException&) {
            throw FaspChannelClosedException();
        }
    }

    void close() {
        _remoteFaspstreamSocket.close();
    }
        
};


static
void
setupStreams() {
#ifdef WIN32
    /*
     * Required on Windows for binary files
     */
    _setmode(fileno(stdin), O_BINARY);
    _setmode(fileno(stdout), O_BINARY);
#endif
    std::cin.exceptions( std::ifstream::badbit );
    std::cout.exceptions( std::ifstream::badbit );
}

static
void
log_msg(const char *msg)
{
    fprintf(stderr, "%s\n", msg);
}

int main()
{
    char data[512];
    setupStreams();
    try {
        /*
         * Set up regular TCP connection with server.
         *
         * IMPORTANT: The destructor for your implementation of
         *            FaspConnectionChannel must fire *after* the
         *            destructor for the FaspStreamClient. Otherwise,
         *            you may get a seg fault in the background thread.
         *
         *            This example will work properly because destructors
         *            for variables on the stack are fired in the reverse
         *            order they are allocated, in C++.
         */
        std::cout << "Connecting to server...\n";
        SocketForwardingChannel sf(REMOTE_HOST, REMOTE_PORT);
        std::cout << "Client connected, setting up faspstream tunneling...\n";
        /*
         * Initialize faspstream client
         */
        FaspStreamClient fsc("Unique-ID-For-a-Client",
            dynamic_cast<FaspConnectionChannel *>(&sf));
        /*
         * Configure client
         * ----------------
         *
         * NOTE: We do not need to configure the TCP port since we are
         * using out-of-band/tunneling.
         *
         * Set target rate to 100 Mbps
         */
        fsc.setRateTarget(100000);
        /*
         * Set the UDP Port for the fasp transfer
         */
        fsc.setUdpPort(33001);

        fsc.setTransferMode(FASPSTREAM_TRANSFER_MODE_TYPE_DIRECT_SEND);

        /*
         * Register logging function callback
         */
        fsc.registerLogCallback(log_msg);

        /*
         * Connect to server
         * -----------------
         *
         * Host should be the remote peer where the remote faspstream is 
         * running. This is most likely the same host/ip that you used
         * to create the regular TCP socket.
         */
        fsc.connect(REMOTE_HOST);
        std::cerr << "Connected to server, sending data..."
                  << std::endl;
        /*
         * Send bytes from stdin
         */

        long totalSent = 0;
        while (std::cin) {
            long sent;
            /*
             * Read from stdin
             */
            std::cin.read(data, sizeof(data));
            sent = std::cin.gcount();
            totalSent += sent;
            /*
             * Write to fasp stream
             */
            fsc.getFaspOutputStream().write(data,
                static_cast<int>(sent));

            std::cerr << "Bytes Sent: " << totalSent << "\r";
            std::cerr.flush();
        }
        /*
         * Done; clean up client.
         */
        fsc.cleanup();

        std::cerr << std::endl 
                  << "Transfer Complete!" 
                  << std::endl;
        std::cerr << "Bytes Sent: " 
                  << totalSent 
                  << std::endl;
        /*
         * Tear down SocketForwardingChannel
         */
        sf.close();

    }
    catch (std::ifstream::failure &e) {
        std::cerr << "Error reading from stdin: "
                  << std::endl
                  << e.what();
    } 
    catch (std::exception &e) {
            std::cerr << "A standard exception occurred:" << std::endl;
            std::cerr << e.what();
    } 
    catch (...)  {
        std::cerr << "An unknown exception ocurred! ";
        std::cerr << "(No further information is available)";
    }

    return 0;
}
