/*
 * Aspera, Inc.
 *
 * Implementation of wrapper in poco_stream_socket.h. Use
 * the interface there.
 */

#include <iostream>
#include <string>
#include <Poco/Exception.h>
#include <Poco/Net/SocketAddress.h>
#include <Poco/Net/ServerSocket.h>
#include <Poco/Net/StreamSocket.h>
#include <Poco/Net/SocketStream.h>
#include <Poco/SharedPtr.h>

#include "poco_stream_socket.h"

struct poco_stream_socket {

    poco_stream_socket(const Poco::Net::StreamSocket& poco_socket) :
        poco_socket(new Poco::Net::StreamSocket(poco_socket)) {}
    ~poco_stream_socket() {
        delete this->poco_socket;
    }
        
    Poco::Net::StreamSocket *poco_socket;
};

extern "C"
int
poco_stream_socket_client(const char *host, int port, poco_stream_socket_t **s)
{
    /*
     * Open connection.
     */
    try {
        Poco::Net::SocketAddress sa(host, port);
        *s = new poco_stream_socket(Poco::Net::StreamSocket(sa));
    }
    catch (Poco::Exception& e) {
        std::cerr << "Error opening client socket: " 
                  << e.displayText() 
                  << std::endl;
        return -1;
    }

    return 0;
}

extern "C"
int
poco_stream_socket_server(int port, poco_stream_socket_t **s)
{
    try {
        /*
         * Set up server socket to listen on port; does bind and listen.
         * Will close once we go out of scope.
         */
        Poco::Net::ServerSocket srv(port);
        /*
         * Listen for incoming client.
         */
        *s = new poco_stream_socket(srv.acceptConnection());
    }
    catch (Poco::Exception& e) {
        std::cerr << "Error opening server socket: " 
                  << e.displayText() 
                  << std::endl;
        return -1;
    }

    return 0;
}

extern "C"
int
poco_stream_socket_destroy(poco_stream_socket_t *s)
{
    /*
     * Delete s (will delete underlying Poco socket)
     */
    delete s;

    return 0;
}

extern "C"
int
poco_stream_socket_send_bytes(poco_stream_socket_t *s, const void *buff,
    int len, int flags, int *ret)
{
    try {
        /*
         * Call sendBytes on poco socket
         */
        *ret = s->poco_socket->sendBytes(buff, len, flags);
    }
    catch (Poco::Exception& e) {
        std::cerr << "Error writing to socket: " 
                  << e.displayText() 
                  << std::endl;
        return -1;
    }
    return 0;
}

extern "C"
int
poco_stream_socket_receive_bytes(poco_stream_socket_t *s, void *buff, int len, 
    int flags, int *ret, int *timedout)
{
    try {
        /*
         * Call receiveBytes on poco socket
         */
        *ret = s->poco_socket->receiveBytes(buff, len, flags);
    }
    catch (Poco::TimeoutException&) {
        /*
         * Timed out; set flag and return.
         */
        *ret = 0;
        *timedout = 1;
        return 0;
    }
    catch (Poco::Exception& e) {
        std::cerr << "Error reading from socket: " 
                  << e.displayText() 
                  << std::endl;
        return -1;
    }
    return 0;
}

extern "C"
void
poco_stream_socket_set_receive_timeout(poco_stream_socket_t *s,
    int microseconds)
{
    s->poco_socket->setReceiveTimeout(microseconds);
}
