/*
 * Aspera, Inc.
 *
 * This is a quick wrapper around C++ Poco sockets. We use it
 * to:
 *     -- create a server socket that listens for *one* client
 *        connection
 *     -- create a client socket that connects to the server
 *
 * To open a client socket:
 *
 *    poco_stream_socket_t *s;
 *    int ret;
 *
 *    ret = poco_stream_socket_client(host, port, &s);
 *
 *    ret = poco_stream_socket_send_bytes(s, buff, length, flags);
 *
 *    ret = poco_stream_socket_receive_bytes(s, buff, length, flags);
 *
 *    poco_stream_socket_destroy(s);
 *
 * To open a server socket:
 *
 *    poco_stream_socket_t *s;
 *    int ret;
 *
 *    ret = poco_stream_socket_server(port, &s);
 *
 *    ...
 */

#ifndef POCO_STREAM_SOCKET_H
#define POCO_STREAM_SOCKET_H

#ifdef __cplusplus
extern "C" {
#endif

struct poco_stream_socket;

typedef struct poco_stream_socket poco_stream_socket_t;

/**
 * Connect to a server at host:port. Returns non-zero on error.
 *
 * @param host the host to connect to
 * @param port the port to connect to
 * @param s out parameter for new socket
 * @return non-zero on error
 */
int
poco_stream_socket_client(const char *host, int port,
    poco_stream_socket_t **s);

/**
 * Listen for *one* client connection request, and return a Poco
 * StreamSocket for that connection. (This is basically a wrapper
 * around calls to open a Poco ServerSocket, do one accept, and
 * return the Poco StreamSocket for the new connection.)
 *
 * @param port the port to listen on
 * @param s out parameter for new socket
 * @return pointer to new Poco StreamSocket
 */
int
poco_stream_socket_server(int port, poco_stream_socket_t **s);

/**
 * Closes and destroys Poco StreamSocket s.
 *
 * @param s the Poco StreamSocket to destroy
 * @return non-zero on failure
 */
int
poco_stream_socket_destroy(poco_stream_socket_t *s);

/**
 * Wrapper for Poco SteamSocket::sendBytes.
 *
 * @param s the Poco StreamSocket
 * @param buff data to send
 * @param len length of buff
 * @param flags Poco flags
 * @param ret out parameter, return value from sendBytes
 * @return return code from sendBytes
 */
int
poco_stream_socket_send_bytes(poco_stream_socket_t *s, const void *buff,
    int len, int flags, int *ret);

/**
 * Wrapper for Poco SteamSocket::receiveBytes.
 *
 * @param s the Poco StreamSocket
 * @param buff buffer to receive data into
 * @param len length of buff
 * @param flags Poco flags
 * @param ret out parameter, return value from receiveBytes
 * @param timedout out parameter, set to non-zero if the socket read times out
 * @return return code from receiveBytes
 */
int
poco_stream_socket_receive_bytes(poco_stream_socket_t *s, void *buff,
    int len, int flags, int *ret, int *timedout);

/**
 * Wrapper for Poco SteamSocket::setReceiveTimeout.
 *
 * @param s the Poco StreamSocket
 * @param timeout the amount of time to wait for receive, in microseconds
 */
void
poco_stream_socket_set_receive_timeout(poco_stream_socket_t *s,
    int microseconds);

#ifdef __cplusplus
}
#endif

#endif
